//
//  PhobosDevice.m
//  GMPhobos
//
//  Created by Q14 on 2020/9/22.
//

#import "PhobosDevice.h"
#import <AdSupport/ASIdentifierManager.h>
#import <GMCache/GMCache.h>
#import <SAMKeychain/SAMKeychain.h>
#import "PhobosUtil.h"

NSString *const KprivatePolicyNotification = @"gm_popup_private_policy_notification";
//钥匙串账户
NSString *const keyChainAccount = @"com.wanmeizhensuo.ZhengXing.phobos.keychain";
//钥匙串账户 表示device_id
NSString *const DEVICEID = @"device_id";
//钥匙串账户 表示idfa
NSString *const IDFA = @"idfa";
// 默认获取不到的idfa
NSString *defaultIDFA = @"00000000-0000-0000-0000-000000000000";

static dispatch_queue_t queue;

@implementation PhobosDevice

#pragma mark - IDFA
/** iOS 14 适配 ATTrackingManagerAuthorizationStatus */
 //同步获取IDFA 直接从钥匙串中获取
+ (NSString *)getIDFA {
    if (![PhobosDevice showPrivatePolicy]) {
        return defaultIDFA;
    }
    //直接从钥匙串中获取
    NSString *keyChainIDFA = [PhobosDevice fetchObjectAtKeyChainWithkey:IDFA];
    if ([PhobosDevice checkIdfaIsNull:keyChainIDFA]) {
        keyChainIDFA = defaultIDFA;
    }
    return keyChainIDFA;
}

/// 请求获取IDFA授权
/// @param completeBlock 回调执行结果
+ (void)requestTrackingAuthorizaWithCompletionHandler:(PhobosGetIDFACompleteBlock)completeBlock {
    
    // iOS14及以上版本需要先请求权限
    if (@available(iOS 14, *)) {
        [ATTrackingManager requestTrackingAuthorizationWithCompletionHandler:^(ATTrackingManagerAuthorizationStatus status) {
            dispatch_async(dispatch_get_main_queue(), ^{
                NSString *idfaString;
                if (status == ATTrackingManagerAuthorizationStatusAuthorized) {
                    idfaString = [[ASIdentifierManager sharedManager] advertisingIdentifier].UUIDString;
                    if (![PhobosDevice checkIdfaIsNull:idfaString]) {
                        [PhobosDevice storeObjectAtKeyChainWithkey:IDFA object:idfaString];
                    }
                } else {
                    // NSLog(@"请在设置-隐私-跟踪中允许App请求跟踪");
                }
                if (completeBlock) {
                    completeBlock(idfaString,status);
                }
            });
        }];
    } else {
        // iOS14以下版本依然使用原方式访问IDFA
        // 判断在设置-隐私里用户是否打开了广告跟踪
        NSInteger status = 0;
        NSString *idfaString;
        if ([[ASIdentifierManager sharedManager] isAdvertisingTrackingEnabled]) {
            idfaString = [[[ASIdentifierManager sharedManager] advertisingIdentifier] UUIDString];
            if (![PhobosDevice checkIdfaIsNull:idfaString]) {
                // 标示 用户已授权打开
                status = 3;
                [PhobosDevice storeObjectAtKeyChainWithkey:IDFA object:idfaString];
            }
        } else {
            // 表示用户拒绝打开
            status = 2;
            // NSLog(@"请在设置-隐私-广告中打开广告跟踪功能");
        }
        if (completeBlock) {
            completeBlock(idfaString,status);
        }
    }
}

#pragma mark - 对IDFA判空处理
//检查idfa 是否为空
+ (BOOL)checkIdfaIsNull:(NSString *)idfa {
    return !idfa.length || [idfa isEqualToString:defaultIDFA];
}

#pragma mark - IDFV
+ (NSString *)getIDFV {
    if (![PhobosDevice showPrivatePolicy]) {
        return defaultIDFA;
    }
    return [[[UIDevice currentDevice] identifierForVendor] UUIDString];
}

#pragma mark - DEVICEID
+ (NSString *)getDEVICEID {
    if (![PhobosDevice showPrivatePolicy]) {
        return defaultIDFA;
    }
    NSString *idfa = [self getIDFA];
    if ([self checkIdfaIsNull:idfa]) {
        idfa = [self getUQID];
    }
    return idfa;
}

#pragma mark - UQID
+ (NSString *)getUQID {
    if (![PhobosDevice showPrivatePolicy]) {
        return defaultIDFA;
    }
    NSString *deviceId = [PhobosDevice fetchObjectAtKeyChainWithkey:DEVICEID];
    if ([PhobosDevice checkIdfaIsNull:deviceId]) {
        deviceId = [self getIDFV];
        [PhobosDevice storeObjectAtKeyChainWithkey:DEVICEID object:deviceId];
    }
    return deviceId;
}

#pragma mark - 钥匙串封装
+ (void)saveKeyChainWithValue:(NSString *)value service:(NSString *)service account:(NSString *)account {
    //避免在重复相同的key 重复设置
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        queue = dispatch_queue_create("com.device.idfa.serial.queue", DISPATCH_QUEUE_SERIAL);
    });
    
    if (queue && [PhobosUtil isNonEmpty:service] && ![self checkIdfaIsNull:value]) {
        dispatch_sync(queue, ^{
            [SAMKeychain setPassword:value forService:service account:account];
            [SAMKeychain setAccessibilityType:kSecAttrAccessibleAlwaysThisDeviceOnly];
        });
    }
}

/***  @brief  给特定的键，标记钥匙串数据并存储 */
+ (void)storeObjectAtKeyChainWithkey:(NSString *)key object:(id <NSCoding>)object {
    NSAssert([PhobosDevice gmKeyChain_isNonEmpty:key], @"key不能为空");
    if (![PhobosDevice gmKeyChain_isNonEmpty:key]) {
        return;
    }
    [self saveKeyChainWithValue:object service:key account:keyChainAccount];
}

/***  @brief  给特定的键，获取对于的钥匙串数据 */
+ (id)fetchObjectAtKeyChainWithkey:(NSString *)key {
    NSAssert([PhobosDevice gmKeyChain_isNonEmpty:key], @"key不能为空");
    if (![PhobosDevice gmKeyChain_isNonEmpty:key]) {
        return nil;
    }
    return [SAMKeychain passwordForService:key account:keyChainAccount];
}

/***  @brief  删除特定的键,对应的钥匙串数据 */
+ (void)removeObjectAtKeyChainWithkey:(NSString *)key {
    NSAssert([PhobosDevice gmKeyChain_isNonEmpty:key], @"key不能为空");
    if (![PhobosDevice gmKeyChain_isNonEmpty:key]) {
        return;
    }
    [SAMKeychain deletePasswordForService:key account:keyChainAccount];
}

+ (BOOL)gmKeyChain_isNonEmpty:(NSString *)key {
    
    NSMutableCharacterSet *emptyStringSet = [[NSMutableCharacterSet alloc] init];
    [emptyStringSet formUnionWithCharacterSet: [NSCharacterSet whitespaceAndNewlineCharacterSet]];
    [emptyStringSet formUnionWithCharacterSet: [NSCharacterSet characterSetWithCharactersInString: @"　"]];
    if ([key length] == 0) {
        return NO;
    }
    NSString* str = [key stringByTrimmingCharactersInSet:emptyStringSet];
    return [str length] > 0;
}

+ (BOOL)showPrivatePolicy {
    return [GMCache fetchObjectAtDocumentPathWithkey:KprivatePolicyNotification];
}

@end
